
# Mindex

A small simple javascript compound index that borrows heavily from LokiJS.

### Features

- **Small** - Under 300 lines of code.
- **Fast** - Records can be found in O(log n) time.
- **Powerful** - Supports compound keys and allows for a simple query syntax.

[![npm version](https://img.shields.io/npm/v/mindex.svg)](https://www.npmjs.com/package/mindex) [![license](https://img.shields.io/npm/l/mindex.svg)](https://github.com/internalfx/mindex/blob/master/LICENSE) [![Build Status](https://travis-ci.org/internalfx/mindex.svg?branch=master)](https://travis-ci.org/internalfx/mindex)

[![NPM](https://nodei.co/npm/mindex.png?downloads=true&downloadRank=true&stars=true)](https://npmjs.org/package/mindex)

---

Special thanks to [Arthur Andrew Medical](http://www.arthurandrew.com/) for sponsoring this project.

Arthur Andrew Medical manufactures products with ingredients that have extensive clinical research for safety and efficacy. We specialize in Enzymes, Probiotics and Antioxidants.

---

## Installation

Supports node v4.0+

```
npm install --save mindex
```

## TL;DR

```javascript
var Mindex = require('mindex')

var index = Mindex(['age'])

index.insertRecord({
  id: 'John',
  age: 25
})
index.insertRecord({
  id: 'Darcy',
  age: 28
})
index.insertRecord({
  id: 'Jim',
  age: 29
})
index.insertRecord({
  id: 'Betty',
  age: 25
})

// Get IDs by key
console.log(index.get(25)) // [ 'Betty', 'John' ]

// Get all IDs sorted by key (age)
console.log(index.getAll()) // [ 'Betty', 'John', 'Darcy', 'Jim' ]

// Get all IDs within a given range
console.log(index.query({'>': [22], '<': [29]})) // [ 'Betty', 'John', 'Darcy' ]

```

---

### `Constructor(keyList)`

##### Parameters

| key | default | type | description |
| --- | --- | --- | --- |
| keyList | [] | Array | The array of keys to index records by |

##### returns

`mindex instance`

##### Description

Creates a new index.

##### Example

```javascript
var Mindex = require('mindex')

var mindex = Mindex(['city', 'age'])
```

---

### `insertRecord(data)`

##### Parameters

| key | default | type | description |
| --- | --- | --- | --- |
| data | undefined | Object | The object to index. Object must have an `id` property |

##### returns

undefined

##### Description

Inserts a new object into the index.

##### Example

```javascript
mindex.insertRecord({
  id: 25,
  name: 'John',
  city: 'Denver',
  age: 35
})
```

---

### `removeRecord(data)`

##### Parameters

| key | default | type | description |
| --- | --- | --- | --- |
| data | undefined | Object | The object to remove. Object must have an `id` property |

##### returns

undefined

##### Description

Removes an object from the index.

##### Example

```javascript
mindex.removeRecord({
  id: 25,
  name: 'John',
  city: 'Denver',
  age: 35
})
```

---

### `updateRecord(data)`

##### Parameters

| key | default | type | description |
| --- | --- | --- | --- |
| data | undefined | Object | The object to update. Object must have an `id` property |

##### returns

undefined

##### Description

Replaces an object in the index with new data.

##### Example

```javascript
mindex.updateRecord({
  id: 25,
  name: 'John',
  city: 'Denver',
  age: 35
})
```

---

### `get(keyList)`

##### Parameters

| key | default | type | description |
| --- | --- | --- | --- |
| keyList | undefined | Mixed | An array of values to retrieve from the index. If the index is not compound a string or integer may be used. |

##### returns

An array of found record IDs.

##### Description

Retrieves an array of matching IDs from the index.

##### Example

```javascript

mindex.insertRecord({
  id: 1,
  name: 'John',
  city: 'Denver',
  age: 35
})

mindex.insertRecord({
  id: 2,
  name: 'Betty',
  city: 'Denver',
  age: 37
})

// Get all 35 year olds in the city of Denver
mindex.get(['Denver', 35]) // returns [1]

// If you are only looking for 'Denver' you can omit the array.
mindex.get('Denver') // returns [1, 2]
```

---

### `getAll()`

##### returns

An array of all IDs in the index, sorted by key(s).

##### Description

Retrieves an array of all IDs.

##### Example

```javascript

mindex.insertRecord({
  id: 1,
  name: 'John',
  city: 'Denver',
  age: 35
})

mindex.insertRecord({
  id: 2,
  name: 'Betty',
  city: 'Denver',
  age: 37
})

mindex.getAll() // returns [1, 2]
```

---

### `query(query)`

##### Parameters

| key | default | type | description |
| --- | --- | --- | --- |
| query | undefined | Object | An object representing the query. |

##### Query Options

| key | default | type | description |
| --- | --- | --- | --- |
| `>` | undefined | Array | An array of keys to search for. Mutually exclusive with `>=` |
| `>=` | undefined | Array | An array of keys to search for. Mutually exclusive with `>` |
| `<` | undefined | Array | An array of keys to search for. Mutually exclusive with `<=` |
| `<=` | undefined | Array | An array of keys to search for. Mutually exclusive with `<` |
| `offset` | 0 | Integer | Number of IDs to skip, can be used for pagination. |
| `limit` | undefined | Integer | Maximum number of IDs to return |

##### returns

An array of found record IDs.

##### Description

Retrieves an array of matching IDs from the index.

##### Example

```javascript

var mindex = Mindex(['city', 'age'])

mindex.insertRecord({
  id: 1,
  name: 'John',
  city: 'Denver',
  age: 35
})

mindex.insertRecord({
  id: 2,
  name: 'Betty',
  city: 'Denver',
  age: 37
})

mindex.insertRecord({
  id: 3,
  name: 'Jen',
  city: 'Phoenix',
  age: 36
})

mindex.insertRecord({
  id: 4,
  name: 'Jim',
  city: 'Denver',
  age: 37
})

index.query({'>': ['Denver', 35], '<=': ['Denver', 37]}) // returns [2,4]

index.query({'>=': ['Denver', 35], '<=': ['Denver', 37], offset: 1}) // returns [2,4]

index.query({'>=': ['Denver', 35], '<=': ['Denver', 37], limit: 1}) // returns [1,2]

index.query({'>': ['Denver'], '<=': ['Phoenix']}) // returns [3]

index.query({'>=': ['Denver'], '<=': ['Phoenix']}) // returns [1,2,4,3]

index.query({'>=': ['Denver'], '<': ['Phoenix']}) // returns [1,2,4]

index.query({'>=': ['Denver']}) // returns [1,2,4,3]
```

---

### `clear()`

##### returns

undefined

##### Description

Clears the index.

##### Example

```javascript

mindex.insertRecord({
  id: 1,
  name: 'John',
  city: 'Denver',
  age: 35
})

mindex.insertRecord({
  id: 2,
  name: 'Betty',
  city: 'Denver',
  age: 37
})

mindex.clear()

mindex.getAll() // returns []
```
